@@ -1,8 +1,30 @@
 use indoc::indoc;
 use insta::assert_snapshot;
 
+use super::assert_valid_toml;
 use crate::{format_toml, Settings};
 
+fn default_settings() -> Settings {
+    Settings {
+        column_width: 120,
+        indent: 2,
+        keep_full_version: false,
+        max_supported_python: (3, 9),
+        min_supported_python: (3, 9),
+        generate_python_version_classifiers: false,
+        table_format: String::from("short"),
+        expand_tables: vec![],
+        collapse_tables: vec![],
+    }
+}
+
+fn long_format_settings() -> Settings {
+    Settings {
+        table_format: String::from("long"),
+        ..default_settings()
+    }
+}
+
 fn format_toml_helper(
     start: &str,
     indent: usize,
@@ -11,17 +33,15 @@ fn format_toml_helper(
     generate_python_version_classifiers: bool,
 ) -> String {
     let settings = Settings {
-        column_width: 120,
         indent,
         keep_full_version,
         max_supported_python,
-        min_supported_python: (3, 9),
         generate_python_version_classifiers,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
+        ..default_settings()
     };
-    format_toml(start, &settings)
+    let result = format_toml(start, &settings);
+    assert_valid_toml(&result);
+    result
 }
 
 #[test]
@@ -43,27 +63,29 @@ fn test_format_toml_simple() {
     let res = format_toml_helper(start, 2, false, (3, 13), true);
     assert_snapshot!(res, @r#"
     # comment
-    a= "b"
+    a = "b"
 
     [build-system]
-    build-backend="backend"
-    requires=["c>=1.5", "d==2"]
+    build-backend = "backend"
+    requires = [ "c>=1.5", "d==2" ]
 
     [project]
-    name="alpha"
+    name = "alpha"
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
       "Programming Language :: Python :: 3.12",
       "Programming Language :: Python :: 3.13",
-    ]dependencies=["e>=1.5"]
+    ]
+    dependencies = [ "e>=1.5" ]
 
     [dependency-groups]
-    test=["p>1"]
+    test = [ "p>1" ]
 
     [tool.mypy]
-    mk="mv"
+    mk = "mv"
     "#);
 }
 
@@ -186,25 +208,14 @@ fn test_table_format_long_with_entry_points() {
         entry-points."console_scripts".mycli = "pkg:main"
         entry-points."console_scripts".othercli = "pkg:other"
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
-    assert_snapshot!(got, @r###"
-        [project]
-        name = "example"
-        [project.entry-points]
-        "console_scripts".mycli = "pkg:main"
-        "console_scripts".othercli = "pkg:other"
-        "###);
+    let got = format_toml(start, &long_format_settings());
+    assert_snapshot!(got, @r#"
+    [project]
+    name = "example"
+    [project.entry-points]
+    "console_scripts".mycli = "pkg:main"
+    "console_scripts".othercli = "pkg:other"
+    "#);
 }
 
 /// Test expand_tables with project.authors
@@ -219,15 +230,8 @@ fn test_expand_project_authors() {
         ]
         "#};
     let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
         expand_tables: vec![String::from("project.authors")],
-        collapse_tables: vec![],
+        ..default_settings()
     };
     let got = format_toml(start, &settings);
     assert_snapshot!(got, @r#"
@@ -256,15 +260,8 @@ fn test_expand_project_maintainers() {
         ]
         "#};
     let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
         expand_tables: vec![String::from("project.maintainers")],
-        collapse_tables: vec![],
+        ..default_settings()
     };
     let got = format_toml(start, &settings);
     assert_snapshot!(got, @r#"
@@ -292,15 +289,8 @@ fn test_expand_single_author() {
         ]
         "#};
     let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
         expand_tables: vec![String::from("project.authors")],
-        collapse_tables: vec![],
+        ..default_settings()
     };
     let got = format_toml(start, &settings);
     assert_snapshot!(got, @r#"
@@ -326,18 +316,7 @@ fn test_collapse_authors_with_url_field() {
         name = "Alice"
         email = "alice@example.com"
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     assert_snapshot!(got, @r#"
     [project]
     name = "test"
@@ -356,18 +335,7 @@ fn test_collapse_empty_authors() {
         [[project.authors]]
         [[project.authors]]
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     assert_snapshot!(got, @r#"
     [project]
     name = "test"
@@ -382,18 +350,7 @@ fn test_collapse_empty_authors() {
 #[test]
 fn test_collapse_authors_without_trailing_newline() {
     let start = "[project]\nname = \"test\"\n[[project.authors]]\nname = \"Alice\"\nemail = \"alice@example.com\"";
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     assert!(got.contains("authors = ["));
     assert!(got.contains("{ name = \"Alice\", email = \"alice@example.com\" }"));
 }
@@ -403,18 +360,7 @@ fn test_collapse_authors_without_trailing_newline() {
 fn test_collapse_authors_compact_parent() {
     let start =
         "[project]\nname=\"test\"\nversion=\"1.0\"\n[[project.authors]]\nname=\"Alice\"\nemail=\"alice@example.com\"";
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     assert!(got.contains("authors = ["));
 }
 
@@ -429,15 +375,8 @@ fn test_expand_authors_already_expanded() {
         email = "john@example.com"
         "#};
     let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
         expand_tables: vec![String::from("project.authors")],
-        collapse_tables: vec![],
+        ..long_format_settings()
     };
     let got = format_toml(start, &settings);
     assert!(got.contains("[[project.authors]]"));
@@ -614,18 +553,7 @@ fn test_no_duplicate_requires() {
         build-backend = "backend"
         requires = ["c", "d"]
     "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     let count = got.matches("requires").count();
     assert_eq!(count, 1, "requires should appear exactly once, but got:\n{}", got);
 }
@@ -642,18 +570,7 @@ fn test_table_format_long_removes_blank_lines_between_same_group() {
         [project.optional-dependencies]
         dev = ["pytest"]
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &long_format_settings());
     assert_snapshot!(got, @r#"
     [project]
     name = "test"
@@ -676,18 +593,7 @@ fn test_table_format_long_with_tool_tables() {
         [tool.mypy]
         strict = true
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &long_format_settings());
     assert_snapshot!(got, @r#"
     [tool.ruff]
     line-length = 120
@@ -708,18 +614,7 @@ fn test_table_format_long_preserves_blank_lines_between_different_groups() {
         [project]
         name = "test"
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &long_format_settings());
     assert_snapshot!(got, @r#"
     [build-system]
     requires = [ "setuptools" ]
@@ -761,18 +656,7 @@ fn test_extract_table_names_from_array_tables() {
 #[test]
 fn test_format_with_trailing_newline_preserved() {
     let start = "[project]\nname = \"test\"\n";
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("short"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &default_settings());
     assert_snapshot!(got, @r#"
     [project]
     name = "test"
@@ -788,18 +672,7 @@ fn test_tool_prefix_extraction_with_dotted_keys() {
         [tool.coverage.report]
         precision = 2
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &long_format_settings());
     assert_snapshot!(got, @"
     [tool.coverage.report]
     precision = 2
@@ -833,18 +706,7 @@ fn test_format_with_non_table_lines_between_headers() {
         [project.urls]
         homepage = "https://example.com"
         "#};
-    let settings = Settings {
-        column_width: 120,
-        indent: 2,
-        keep_full_version: false,
-        max_supported_python: (3, 9),
-        min_supported_python: (3, 9),
-        generate_python_version_classifiers: false,
-        table_format: String::from("long"),
-        expand_tables: vec![],
-        collapse_tables: vec![],
-    };
-    let got = format_toml(start, &settings);
+    let got = format_toml(start, &long_format_settings());
     assert_snapshot!(got, @r#"
     [project]
     name = "test"
@@ -912,3 +774,49 @@ fn test_lib_module_registration() {
         assert!(module.hasattr("Settings").unwrap());
     });
 }
+
+#[test]
+fn test_idempotent_formatting() {
+    let start = indoc! {r#"
+        [project]
+        name = "test"
+        description = "This is a long description string that needs to exceed the default column width of one hundred and twenty characters to trigger wrapping."
+    "#};
+    let settings = Settings {
+        column_width: 120,
+        indent: 2,
+        keep_full_version: false,
+        max_supported_python: (3, 9),
+        min_supported_python: (3, 9),
+        generate_python_version_classifiers: false,
+        table_format: String::from("short"),
+        expand_tables: vec![],
+        collapse_tables: vec![],
+    };
+    let first = format_toml(start, &settings);
+    let second = format_toml(&first, &settings);
+    let third = format_toml(&second, &settings);
+    assert_eq!(first, second, "formatting should be idempotent (first->second)");
+    assert_eq!(second, third, "formatting should be idempotent (second->third)");
+}
+
+#[test]
+fn test_issue_186_single_quote_with_comments() {
+    let start = indoc! {r#"
+    [tool.something]
+    items = [
+        'first',
+        # A comment
+        'second',
+    ]
+    "#};
+    let got = format_toml(start, &default_settings());
+    assert_snapshot!(got, @r#"
+    [tool.something]
+    items = [
+      "first",
+      # A comment
+      "second",
+    ]
+    "#);
+}
