@@ -7,7 +7,7 @@ use indoc::indoc;
 use tombi_config::TomlVersion;
 
 use crate::project::fix;
-use crate::tests::{collect_entries, format_syntax, format_toml_str, parse};
+use crate::tests::{assert_valid_toml, collect_entries, format_syntax, format_toml_str, parse};
 use crate::TableFormatConfig;
 
 fn evaluate_project(
@@ -42,15 +42,16 @@ fn evaluate_project(
     let entries = collect_entries(&tables);
     root_ast.splice_children(0..count, entries);
     ensure_all_arrays_multiline(&root_ast, 120);
-    format_syntax(root_ast, 120)
+    let result = format_syntax(root_ast, 120);
+    assert_valid_toml(&result);
+    result
 }
 
 #[test]
 fn test_project_no_project_section() {
     let start = "";
     let result = evaluate_project(start, false, (3, 9), true);
-    insta::assert_snapshot!(result, @"
-");
+    insta::assert_snapshot!(result, @"");
 }
 
 #[test]
@@ -61,10 +62,12 @@ fn test_project_dependencies_normalize_no_keep() {
     "#};
     let result = evaluate_project(start, false, (3, 9), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",][project]
-    dependencies=["a>=1", "b-c>=1.5"]
+      "Programming Language :: Python :: 3.9",
+    ]
+    dependencies = [ "a>=1", "b-c>=1.5" ]
     "#);
 }
 
@@ -76,10 +79,12 @@ fn test_project_dependencies_normalize_keep_version() {
     "#};
     let result = evaluate_project(start, true, (3, 9), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",][project]
-    dependencies=["a>=1.0.0", "b-c>=1.5.0"]
+      "Programming Language :: Python :: 3.9",
+    ]
+    dependencies = [ "a>=1.0.0", "b-c>=1.5.0" ]
     "#);
 }
 
@@ -93,17 +98,16 @@ fn test_project_optional_dependencies() {
     let result = evaluate_project(start, false, (3, 13), true);
     insta::assert_snapshot!(result, @r#"
     [project]
-
-
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
       "Programming Language :: Python :: 3.12",
       "Programming Language :: Python :: 3.13",
-    ]optional-dependencies.dev = ["pytest>=7"]
-
-    optional-dependencies.docs = ["sphinx>=4"]
+    ]
+    optional-dependencies.dev = [ "pytest>=7" ]
+    optional-dependencies.docs = [ "sphinx>=4" ]
     "#);
 }
 
@@ -215,9 +219,8 @@ fn test_project_authors_maintainers() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
-    maintainers = [ { email = "maintain@example.com" } ]
-
     [project]
+    maintainers = [ { email = "maintain@example.com" } ]
     authors = [ { name = "Jane Smith" }, { name = "John Doe", email = "john@example.com" } ]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
@@ -282,7 +285,7 @@ fn test_project_entry_points() {
       "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
     ]
-    entry-points.mytool = "mypackage:main"
+    entry-points."console_scripts".mytool = "mypackage:main"
     "#);
 }
 
@@ -316,13 +319,13 @@ fn test_project_gui_scripts() {
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
     [project]
-
-
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ]gui-scripts.mygui = "mypackage.gui:main"
+    ]
+    gui-scripts.mygui = "mypackage.gui:main"
     "#);
 }
 
@@ -334,12 +337,14 @@ fn test_project_dynamic_fields() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dynamic = ["description","version" ]
+    ]
+    dynamic = [ "description", "version" ]
     "#);
 }
 
@@ -362,10 +367,12 @@ fn test_project_full_metadata() {
     requires-python = ">=3.9"
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
       "Programming Language :: Python :: 3.12",
-    ]dependencies = ["requests>=2.28"]
+    ]
+    dependencies = [ "requests>=2.28" ]
     "#);
 }
 
@@ -403,17 +410,15 @@ fn test_project_optional_dependencies_multiple_groups() {
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
     [project]
-
-
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ]optional-dependencies.dev = ["black>=22","pytest>=7" ]
-
-    optional-dependencies.docs = ["myst-parser>=0.18","sphinx>=4" ]
-
-    optional-dependencies.test = ["coverage>=6"]
+    ]
+    optional-dependencies.dev = [ "black>=22", "pytest>=7" ]
+    optional-dependencies.docs = [ "myst-parser>=0.18", "sphinx>=4" ]
+    optional-dependencies.test = [ "coverage>=6" ]
     "#);
 }
 
@@ -510,8 +515,8 @@ fn test_project_multiple_entry_point_groups() {
       "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
     ]
-    entry-points.myplugin = "mypackage.plugin:pytest_plugin"
-    entry-points.mytool = "mypackage:main"
+    entry-points."console_scripts".mytool = "mypackage:main"
+    entry-points."pytest11".myplugin = "mypackage.plugin:pytest_plugin"
     "#);
 }
 
@@ -545,12 +550,14 @@ fn test_project_dependencies_with_extras() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["click[colorama]>=8","requests[security]>=2.28" ]
+    ]
+    dependencies = [ "click[colorama]>=8", "requests[security]>=2.28" ]
     "#);
 }
 
@@ -565,15 +572,14 @@ fn test_project_dependencies_with_markers() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = [
-        "importlib-metadata>=4; python_version<'3.10'",
-        "typing-extensions>=4; python_version<'3.11'"
     ]
+    dependencies = [ "importlib-metadata>=4; python_version<'3.10'", "typing-extensions>=4; python_version<'3.11'" ]
     "#);
 }
 
@@ -596,7 +602,7 @@ fn test_project_urls_multiple() {
       "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
     ]
-    urls = "https://github.com/example/repo/issues"
+    urls."Bug Tracker" = "https://github.com/example/repo/issues"
     urls.Changelog = "https://github.com/example/repo/blob/main/CHANGELOG.md"
     urls.Documentation = "https://docs.example.com"
     urls.Homepage = "https://example.com"
@@ -638,11 +644,13 @@ fn test_project_empty_dependencies() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
+    ]
     dependencies = []
     "#);
 }
@@ -656,13 +664,13 @@ fn test_project_empty_optional_dependencies() {
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
     [project]
-
-
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ]optional-dependencies.dev = []
+    ]
+    optional-dependencies.dev = []
     "#);
 }
 
@@ -674,12 +682,14 @@ fn test_project_normalize_package_name_underscores() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["another-package>=2","my-package>=1" ]
+    ]
+    dependencies = [ "another-package>=2", "my-package>=1" ]
     "#);
 }
 
@@ -691,12 +701,14 @@ fn test_project_dependencies_git_urls() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["pkg @ git+https://github.com/user/repo.git@main"]
+    ]
+    dependencies = [ "pkg @ git+https://github.com/user/repo.git@main" ]
     "#);
 }
 
@@ -708,12 +720,14 @@ fn test_project_dependencies_local_paths() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["pkg @ file:///path/to/package"]
+    ]
+    dependencies = [ "pkg @ file:///path/to/package" ]
     "#);
 }
 
@@ -811,23 +825,23 @@ fn test_project_all_fields() {
     version = "1.0.0"
     description = "A comprehensive test"
     readme = "README.md"
-    keywords = ["example","test" ]
+    keywords = [ "example", "test" ]
     license = { text = "MIT" }
-    maintainers = [{ name = "Maintainer" }]
-    authors = [{ name = "Dev", email = "dev@example.com" }]
+    maintainers = [ { name = "Maintainer" } ]
+    authors = [ { name = "Dev", email = "dev@example.com" } ]
     requires-python = ">=3.9"
     classifiers = [
-    "Development Status :: 4 - Beta",
+      "Development Status :: 4 - Beta",
       "Programming Language :: Python :: 3 :: Only",
       "Programming Language :: Python :: 3.9",
       "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
       "Programming Language :: Python :: 3.12",
     ]
-    dependencies = ["requests>=2.28"]
-    optional-dependencies.dev = ["pytest>=7"]
-
-    urls.Homepage = "https://example.com"scripts.mytool = "mypackage:main"
+    dependencies = [ "requests>=2.28" ]
+    optional-dependencies.dev = [ "pytest>=7" ]
+    urls.Homepage = "https://example.com"
+    scripts.mytool = "mypackage:main"
     "#);
 }
 
@@ -906,13 +920,15 @@ fn test_project_dependencies_url_format() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
+    ]
     dependencies = [
-        "pkg @ https://example.com/pkg-1.0.tar.gz",
+      "pkg @ https://example.com/pkg-1.0.tar.gz",
     ]
     "#);
 }
@@ -927,12 +943,13 @@ fn test_project_entry_points_inline_tables() {
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
     [project]
-
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ]entry-points = [{ name = "console_scripts", value = { mytool = "mypackage:main", another = "mypackage:other" } }]
+    ]
+    entry-points = [ { name = "console_scripts", value = { mytool = "mypackage:main", another = "mypackage:other" } } ]
     "#);
 }
 
@@ -944,11 +961,13 @@ fn test_project_scripts_inline_table() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
+    ]
     scripts = { mytool = "mypackage:main", helper = "mypackage.cli:run" }
     "#);
 }
@@ -961,11 +980,13 @@ fn test_project_gui_scripts_inline_table() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
+    ]
     gui-scripts = { mygui = "mypackage.gui:main" }
     "#);
 }
@@ -1183,15 +1204,17 @@ fn test_project_dependencies_complex_markers() {
     "#};
     let result = evaluate_project(start, false, (3, 12), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
       "Programming Language :: Python :: 3.12",
-    ][project]
+    ]
     dependencies = [
-        "other>=2; (python_version>='3.10' or sys_platform!='win32')",
-        "pkg>=1; python_version<'3.10' and platform_system=='Linux'",
+      "other>=2; (python_version>='3.10' or sys_platform!='win32')",
+      "pkg>=1; python_version<'3.10' and platform_system=='Linux'",
     ]
     "#);
 }
@@ -1204,12 +1227,14 @@ fn test_project_dependencies_multiple_extras() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["pkg[extra1,extra2,extra3]>=1"]
+    ]
+    dependencies = [ "pkg[extra1,extra2,extra3]>=1" ]
     "#);
 }
 
@@ -1229,7 +1254,8 @@ fn test_project_urls_with_special_chars() {
       "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
     ]
-    urls = "https://github.com/user/repo"
+    urls."Bug Tracker" = "https://github.com/user/repo/issues"
+    urls."Source Code" = "https://github.com/user/repo"
     "#);
 }
 
@@ -1284,9 +1310,11 @@ fn test_project_dynamic_version_only() {
     name = "test"
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ]dynamic = ["version"]
+    ]
+    dynamic = [ "version" ]
     "#);
 }
 
@@ -1298,12 +1326,14 @@ fn test_project_dependencies_duplicate_handling() {
     "#};
     let result = evaluate_project(start, false, (3, 11), true);
     insta::assert_snapshot!(result, @r#"
+    [project]
     classifiers = [
       "Programming Language :: Python :: 3 :: Only",
-      "Programming Language :: Python :: 3.9",  "Programming Language :: Python :: 3.10",
+      "Programming Language :: Python :: 3.9",
+      "Programming Language :: Python :: 3.10",
       "Programming Language :: Python :: 3.11",
-    ][project]
-    dependencies = ["requests>=2.28", "requests>=2.30"]
+    ]
+    dependencies = [ "requests>=2.28", "requests>=2.30" ]
     "#);
 }
 
@@ -1600,7 +1630,8 @@ fn test_project_entry_points_inline_table_expansion() {
     insta::assert_snapshot!(result, @r#"
     [project]
     name = "test"
-    entry-points.console_scripts.bar = "pkg:bar"entry-points.console_scripts.foo = "pkg:main"
+    entry-points.console_scripts.bar = "pkg:bar"
+    entry-points.console_scripts.foo = "pkg:main"
     "#);
 }
 
@@ -1707,7 +1738,7 @@ fn test_project_urls_all_common() {
     insta::assert_snapshot!(result, @r#"
     [project]
     name = "test"
-    urls = "https://github.com/user/repo/issues"
+    urls."Bug Tracker" = "https://github.com/user/repo/issues"
     urls.Changelog = "https://example.com/changelog"
     urls.Documentation = "https://example.com/docs"
     urls.Homepage = "https://example.com"
@@ -1844,7 +1875,7 @@ fn test_project_optional_deps_empty_group() {
     insta::assert_snapshot!(result, @r#"
     [project]
     name = "test"
-    optional-dependencies.dev = [  ]
+    optional-dependencies.dev = []
     "#);
 }
 
